@@ -19,7 +19,7 @@
  * @author Haruki Setoyama <haruki@planewave.org>
  * @copyright Copyright &copy; 2003-2004, Haruki SETOYAMA <haruki@planewave.org>
  * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
- * @version 0.4 $Id: phpoot.php,v 1.29 2004/03/22 11:36:11 haruki Exp $
+ * @version 0.5 $Id: phpoot.php,v 1.31 2004/04/23 05:02:57 haruki Exp $
  * @link http://phpoot.sourceforge.jp/
  * @package phpoot
  */
@@ -41,7 +41,12 @@ define('PHPOOT', true);
  * extention added to compiled template
  */
 @define('PHPOOT_EXT', '.php');
-
+/**
+ * error code
+ */
+define('PHPOOT_ERR_INTERNAL', 1);
+define('PHPOOT_ERR_TEMPLATE', 2);
+define('PHPOOT_ERR_COMPILE_DIR', 4);
 /**
  * PHPOOT Main Class.
  *
@@ -58,7 +63,6 @@ class phpoot extends PEAR {
      * 'force_compile'      when true, template will be compiled everytime
      * 'check_timestamp'    when true, modified time of template will be checked
      * 'debug'              ehwn true, display debug information in escape form <!-- -->
-     * 'pear_error'         when false, false is returned insted of pearError
      * 'mkdir_mode'         mode for mkdir()
      * </pre>
      * @var array
@@ -67,7 +71,6 @@ class phpoot extends PEAR {
             'force_compile'   => false,
             'check_timestamp' => true,
             'debug'           => false,
-            'pear_error'      => true,
             'mkdir_mode'      => 0777
         );
 
@@ -155,22 +158,21 @@ class phpoot extends PEAR {
         $compiled_file = $this->_compiled_dir . '/' . basename($this->_template_file) . PHPOOT_EXT;
         if ($this->option['force_compile'] != false
         || file_exists($compiled_file) != true) {
-            $err = $this->_compile($compiled_file);
-            if ($err !== true) {
-                return $this->raiseError($err);
-            }
+            $err = $this->compile();
+            if ($err !== true) return $err;
         }
         elseif ($this->option['check_timestamp'] != false
-        && $this->_template_readable() == true
-        && $this->_template_update_time() > filemtime($compiled_file)) {
-            $err = $this->_compile($compiled_file);
-            if ($err !== true) {
-                return $this->raiseError($err);
-            }
+        && (int)$this->_template_update_time() > filemtime($compiled_file)) {
+            $err = $this->compile();
+            if ($err !== true) return $err;
         }
 
-        if (! phpoot_display_compiled($compiled_file, $data, $this->option['debug'])) {
-            return $this->raiseError("Error(s) in compiled file '$compiled_file' or in template file '$this->_template_file'. ");
+		$err = phpoot_display_compiled($this->_compiled_file(), $data, $this->option['debug']);
+        if ($err !== true) {
+			$file = $this->_compiled_file();
+            return $this->raiseError(
+					"Error(s) in the compiled file '$file'. ",
+					PHPOOT_ERR_INTERNAL);
         }
         return true;
     }
@@ -192,59 +194,99 @@ class phpoot extends PEAR {
     }
 
     /**
-     * compiles template
+     * compile template file
      *
-     * @access public
-     * @return bool|PEARError
+     * @return bool
      */
     function compile()
     {
-        $compiled_file = $this->_compiled_dir . '/' . basename($this->_template_file) . PHPOOT_EXT;
-        $err = $this->_compile($compiled_file);
-        if ($err !== true) {
-            return $this->raiseError($err);
-        }
-        return true;
-    }
-
-    /**
-     * compile template file
-     *
-     * @param stirng $compiled_file
-     * @access private
-     * @return bool
-     */
-    function _compile($compiled_file)
-    {
-        if (! $this->_template_readable()) {
-            return "Template file '$this->_template_file' does not exists or is not readable. ";
-        }
-        if (! file_exists($compiled_file)) {
-            $dir = dirname($compiled_file);
-            if (! $this->_mkdirs($dir)) {
-                return "Fail to create the directory '$dir' for compiled files.";
+		$template = $this->_get_template();
+        if (PEAR::isError($template)) return $template;
+		
+		$compiled_file = $this->_compiled_file();
+        
+        if (file_exists($compiled_file)) {
+			if (! is_writable($compiled_file)) {
+                return $this->raiseError(
+					"Compiled file '$compiled_file' is not rewritable. ",
+					PHPOOT_ERR_COMPILE_DIR);
             }
-            if (! is_writable($dir)) {
-                return "Directory '$dir' is not writable for compiled file.";
+		} 
+		elseif(is_dir($this->_compiled_dir)) {
+			if (! is_writable($this->_compiled_dir)) {
+				$dir = $this->_compiled_dir;
+                return $this->raiseError(
+					"Directory for compiled files '$dir' is not writable. ",
+					PHPOOT_ERR_COMPILE_DIR);
+			}
+		}
+		else {
+			if (! $this->_mkdirs($this->_compiled_dir)) {
+				$dir = $this->_compiled_dir;
+                return $this->raiseError(
+					"Failed to create the directory for compiled files '$dir'. ",
+					PHPOOT_ERR_COMPILE_DIR);
             }
-        } else {
-            if (! is_writable($compiled_file)) {
-                return "File '$compiled_file' for compiled template file is not rewritable.";
-            }
-        }
-        $template = $this->_get_template();
-        if (! $template) return "Fail to get template file '$this->_template_file'. ";
+		}
 
         $parsed = $this->_parse($template);
-        $parsed = "<?php if(! defined('PHPOOT')) exit(); ?>\n" . $parsed . "<?php \$_check = true; ?>\n"; // <?
-        $fp_w = fopen($compiled_file, 'wb');
-        if (! $fp_w) return "Fail to open compiled file '$compiled_file'. ";
-        if (! fwrite($fp_w, $parsed)) return "Fail to write compiled file '$compiled_file'. ";
-        ftruncate($fp_w, ftell($fp_w));
+        $parsed = "<?php if(! defined('PHPOOT')) exit(); ?>\n" 
+					. $parsed . "<?php \$_check = true; ?>\n"; // <?
+        
+		$fp_w = fopen($compiled_file, 'wb');
+        if (! $fp_w) {
+            return $this->raiseError(
+				"Failed to open the compiled file '$compiled_file'. ",
+				PHPOOT_ERR_INTERNAL);
+        }
+		$err = fwrite($fp_w, $parsed);
+        if (! $err) {
+			fclose($fp_w);
+            return $this->raiseError(
+				"Fail to write compiled file '$compiled_file'. ",
+				PHPOOT_ERR_INTERNAL);
+        }
+        //ftruncate($fp_w, ftell($fp_w));
         fclose($fp_w);
         return true;
     }
+	
+	/**
+     * remove compiled template file
+     *
+     * @return bool
+     */
+	function removeCompiled()
+	{
+		if (! isset($this->_template_file)) {
+		    return $this->raiseError(
+					"Template file is not set. ",
+					PHPOOT_ERR_TEMPLATE);
+		}
+		
+		$compiled_file = $this->_compiled_file();
+        if (! file_exists($compiled_file)) return false;
 
+		if (! unlink($compiled_file)) {
+            return $this->raiseError(
+					"Failed to remove the compiled file '$compiled_file'. ",
+					PHPOOT_ERR_COMPILE_DIR);
+        }
+		return true;
+	}
+
+	/**
+     * get file name for compiled
+     *
+     * @access private
+     * @return string
+     */
+	function _compiled_file()
+	{
+		return $this->_compiled_dir . '/' 
+			. basename($this->_template_file) . PHPOOT_EXT;
+	}
+	
     /**
      * parse template file
      *
@@ -265,10 +307,6 @@ class phpoot extends PEAR {
 
         $this->_handler->init($template);
         $this->_parser->parse($template);
-        if (! empty($this->_handler->error)) {
-            $this->raiseError("Template has some error. ");
-        }
-
         return $this->_handler->getParsed();
     }
 
@@ -281,7 +319,6 @@ class phpoot extends PEAR {
      */
     function _mkdirs($path)
     {
-        if (strlen($path) == 0) return false;
         if (is_dir($path)) return true;
         if (is_file($path)) return false;
         if (! $this->_mkdirs(dirname($path))) return false;
@@ -307,7 +344,14 @@ class phpoot extends PEAR {
      */
     function _template_readable()
     {
-        return is_readable($this->_template_file);
+		if (is_readable($this->_template_file)) {
+		    return true;
+		} else {
+			$file = $this->_template_file;
+			return $this->raiseError(
+					"Template file '$file' does not exists or is not readable. ",
+					PHPOOT_ERR_TEMPLATE);
+		}
     }
 
     /**
@@ -320,29 +364,13 @@ class phpoot extends PEAR {
     {
         $fd = fopen($this->_template_file, 'rb');
         if (! $fd) {
-            $this->raiseError("Fail to open the template file '$this->_template_file'.");
-            return false;
+			$file = $this->_template_file;
+            return $this->raiseError("Fail to open the template file '$file'. ", PHPOOT_ERR_TEMPLATE);
         }
         $contents = fread ($fd, filesize($this->_template_file));
         fclose ($fd);
         return $contents;
     }
-
-    /**
-     * if $this->option['pear_error'] is true, false is returned by raiseError()
-     *
-     * @access private
-     * @return false|PEARError
-     */
-    function raiseError($msg)
-    {
-        if ($this->option['pear_error']) {
-            return parent::raiseError('<b>PHPOOT Error</b>: '.htmlspecialchars($msg));
-        } else {
-            parent::raiseError('<b>PHPOOT Error</b>: '.htmlspecialchars($msg));
-            return false;
-        }
-    }
 }
 
 
@@ -358,9 +386,8 @@ class phpoot extends PEAR {
 function phpoot_display_compiled($compiled_file, $_val0, $_error =false)
 {
     $_check = false; // flag for parse error check
-    $_no_error = true;
     include $compiled_file;
-    return ($_check && $_no_error);
+    return $_check;
 }
 
 // functions below used in compiled template file.
