@@ -2,7 +2,7 @@
 require_once '../phpoot.php';
 $tmpl = new phpoot;
 //$tmpl->setErrorHandling(PEAR_ERROR_PRINT);
-$tmpl->setTemplateFile('./simple_template.html');
+$tmpl->setTemplateFile('./helloworld.html');
 $model_data = array(
   'title' => 'Hello World',
   'body'  => 'PHPOOT is a template engine for PHP'
